"
jv:libgdbs - GNU Debugger Interface Library
Copyright (C) 2015-now Jan Vrany
Copyright (C) 2020-2021 LabWare

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License. 

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
"
"{ Package: 'jv:libgdbs' }"

"{ NameSpace: Smalltalk }"

Object subclass:#GDBOutputFormat
	instanceVariableNames:'format name description'
	classVariableNames:''
	poolDictionaries:'GDBOutputFormats'
	category:'GDB-Core'
!

!GDBOutputFormat class methodsFor:'documentation'!

copyright
"
jv:libgdbs - GNU Debugger Interface Library
Copyright (C) 2015-now Jan Vrany
Copyright (C) 2020-2021 LabWare

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License. 

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
"
!

documentation
"
    GDBOutputFormat is a reification of one of output formats
    defined by GDB. 

    Do not use this class directly in user code, use constants 
    defined `GDBOutputFormats`

    [author:]
        Jan Vrany <jan.vrany@fit.cvut.cz>

    [instance variables:]

    [class variables:]

    [see also:]
        GDBOutputFormats
        https://sourceware.org/gdb/onlinedocs/gdb/Output-Formats.html#Output-Formats

"
! !

!GDBOutputFormat class methodsFor:'initialization'!

initialize
    "Invoked at system start or when the class is dynamically loaded."

    "/ Following hack is here to allow loading from source and from compiled class lib.
    "/ When loading from source, GDBOutputFormats is not yet initialized
    "/ since it requires this class. So we initialize it now when this
    "/ class is definitely present.
    "/ 
    "/ When loading from binary, classes are initialize after all of them
    "/ are present, therefore we do not need to call initialize again.
    "/ Hence the condition.
    FormatHexadecimal isNil ifTrue: [ 
        GDBOutputFormats initialize.
    ].

    "Modified: / 25-01-2018 / 09:29:51 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified (comment): / 26-03-2021 / 13:29:20 / Jan Vrany <jan.vrany@labware.com>"
! !

!GDBOutputFormat class methodsFor:'instance creation'!

format:formatArg name:nameArg description:descriptionArg 
    ^ self new format:formatArg name:nameArg description:descriptionArg 
! !

!GDBOutputFormat class methodsFor:'accessing'!

address
    ^ FormatAddress

    "Created: / 28-11-2018 / 14:00:46 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

all
    ^ GDBOutputFormats all
!

binary
    ^ FormatBinary

    "Created: / 28-11-2018 / 14:00:58 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

characterConstant
    ^ FormatCharacterConstant

    "Created: / 28-11-2018 / 14:01:12 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

floatingPoint
    ^ FormatFloatingPoint

    "Created: / 28-11-2018 / 14:01:20 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

hexadecimal
    ^ FormatHexadecimal

    "Created: / 28-11-2018 / 14:01:35 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

octal
    ^ FormatOctal

    "Created: / 28-11-2018 / 14:01:47 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

paddedHexadecimal
    ^ FormatPaddedHexadecimal

    "Created: / 28-11-2018 / 13:07:31 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

raw    
    ^ FormatRaw

    "Created: / 28-11-2018 / 14:01:58 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

signedDecimal
    ^ FormatSignedDecimal

    "Created: / 28-11-2018 / 14:02:10 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

string
    ^ FormatString

    "Created: / 28-11-2018 / 14:02:20 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

unsignedDecimal
    ^ FormatUnsignedDecimal

    "Created: / 28-11-2018 / 14:02:35 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!GDBOutputFormat methodsFor:'accessing'!

description
    ^ description
!

format
    ^ format
!

name
    ^ name
! !

!GDBOutputFormat methodsFor:'displaying'!

displayString
    ^ name

    "Created: / 26-01-2018 / 00:46:10 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!GDBOutputFormat methodsFor:'initialization'!

format:formatArg name:nameArg description:descriptionArg 
    format := formatArg.
    name := nameArg.
    description := descriptionArg.
! !

!GDBOutputFormat methodsFor:'printing & storing'!

printOn:aStream
    "append a printed representation of the receiver to the argument, aStream"

    super printOn:aStream.
    aStream nextPutAll:'('.
    format printOn:aStream.
    aStream nextPutAll:')'.

    "Modified: / 28-11-2018 / 13:05:45 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!GDBOutputFormat class methodsFor:'documentation'!

version_HG

    ^ '$Changeset: <not expanded> $'
! !


GDBOutputFormat initialize!
